//---------------------------------------------------------
// Header file for a simple logic analyzer
//
// This logic analyzer is implemented in an FPGA that is
// connected up to an Atmel ATmega2561 AVR microcontroller
//
// It is intended to be used as an aid in debugger FPGA
// hardware.  It is 32 bits wide and has a very simple trigger
// mode.
//-------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "LogicAnalyzer.h"
#include "system.h"
/*!
 * \fn void LA_SetTrigger(long tw, long tl)
 * Set the Trigger word in the logic analyzer
 *
 * \param tw pointer to 8 bytes for the trigger word
 */
void LA_SetTrigger(char *tw)
{
	int i;
	volatile unsigned char *pTW;

	pTW = LA_TRIG0_P;
	for(i=0;i<8;++i) *pTW++ = *tw++;
}

void LA_Arm(void)
{
	LA_CSR = BIT(LA_CR_ARM);
}

char *LA_Read(int n,unsigned short *pN)
{
	//----------------------------------------------------
	// read a block of data back from the
	// logic analyzer FIFO
	//
	// parameters:
	//	n.....number of data words to read (32 bits)
	//	pN....pointer to unsigned short that will recieve
	//			the total number of bytes in the buffer
	//			that is returned.
	//
	// return value:
	//	returns a pointer to a buffer that contains the
	// data.  The data is in the following format:
	//	DATA[1:0] <= Number of longs in buffer
	//	DATA[5:2] <= First long sent back
	//	etc.
	//
	// After the buffer is used, it needs to be
	// destroyed by passing it as a parameter to
	// free()
	//---------------------------------------------------

	unsigned short N = sizeof(long) * n + sizeof(short);
	char *pB = malloc(N);
	char *pL = pB;
	*pN = N;
	int *pI = (int *)pL;
	*pI = n;
	n = N-sizeof(short);
	char *pLA = (char *)LA_RAM_P;

	int i;
	pL += sizeof(short);
		
	for(i=0;i<n;++i)
		*pL++ = *pLA;
	return pB;
}

void LA_Reset(void)
{
	LA_CSR = BIT(LA_CR_CLEAR_READ_ADDRESS);
	LA_CSR = BIT(LA_CR_CLEAR_WRITE_ADDRESS);
}

void LA_GetStatus(LA_STATUS *pS)
{
	pS->WordsWritten = LA_WRITEADR;
	pS->BytesRead = LA_READADR;
	pS->State = LA_CSR;
}
