/*----------------------------------------------------------------------------------
 * Synthesizer device driver file
 *
 * Created September 1, 2008 by James Patchell
 * Copyright (c) 2008 by James Patchell
 *
 *----------------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "Synth.h"
#include "system.h"

/*!
 * \fn void SetOscFreq(int n, unsigned long f)
 * Set NCO oscilator frequency
 *
 * \param n Oscilator number to set
 * \param f Frequency to set oscilator to
 */
void SetOscFreq(unsigned short n, unsigned long f)
{
	unsigned long *pOSC = (unsigned long *)SYNTH_OSC_FREQ_P;
	pOSC[n] = f;
}

/*!
 * \fn long GetOscFreq(int n)
 * Get NCO oscilator frequency
 *
 * \param n Oscilator number to get
 * \return returns number reperesenting oscilator frequency
 */
unsigned long GetOscFreq(unsigned short n)
{
	unsigned long *pOSC = (unsigned long *)SYNTH_OSC_FREQ_P;
	return pOSC[n];
}

/*!
 * \fn void EnableSynth(unsigned char e)
 * Enable or Disable Synthesizer Engine
 *
 * \param e e=1::Enable e=0::Disable
 */
void EnableSynth(unsigned char e)
{
	if(e)
		SYNTH_CONTROL_REG |= BIT(SYNTH_CR_SYNTH_EN);
	else
		SYNTH_CONTROL_REG &= ~BIT(SYNTH_CR_SYNTH_EN);
}
