/*@A (C) 1992 Allen I. Holub                                                */
#include <stdio.h>
#include "cur.h"


/*--------------------------------------------------------
 * WINIO.C	Lowest level I/O routines:
 * 	waddch(win,c)  		wgetch(win)
 *	echo()			noecho()
 *	nl()			nonl()
 *	crmode()		nocrmode()
 *--------------------------------------------------------
 */

				/* and map both to \n\r on output */
void echo	(WINDOW *w) {  w->Echo = 1; }
void noecho	(WINDOW *w) {  w->Echo = 0; }
void nl		(WINDOW *w) {  w->Nl   = 1; }
void nonl	(WINDOW *w) {  w->Nl   = 0; }

void crmode(WINDOW *w)
{
      /* setvbuf(stdin, NULL, _IONBF, 0); */ /* Turn off buffering */
    w->Crmode = 1;
}

void nocrmode(WINDOW *w)
{
     /* freopen( "/dev/con", "r", stdin ); */
    w->Crmode = 0;
}

/*--------------------------------------------------------*/

unsigned char *getbuf(WINDOW *win,unsigned char * buf)
{
    /* Get a buffer interactively. ^H is a destructive backspace. This routine
     * is mildly recursive (it's called from wgetch() when Crmode is false.
     * The newline is put into the buffer. Returns it's second argument.
     */

    int  c;
    unsigned char *sbuf = buf;

    win->Crmode = 1;
    while( (c = wgetch(win)) != '\n' && c != '\r' )
    {
	switch( c )
	{
	case '\b':  if( buf > sbuf )
	            {
			--buf;
			wprintw( win, " \b" );
		    }
		    else
		    {
			wprintw( win, " " );
			putchar('\007');
		    }
		    break;

	default:    *buf++ = c;
		    break;
	}
    }
    *buf++ = c   ;	/* Add line terminator (\n or \r) */
    *buf   = '\0';
    win->Crmode = 0   ;
    return sbuf  ;
}

/*--------------------------------------------------------*/

int	wgetch( WINDOW	*win )
{
    /* Get a character from DOS without echoing. We need to do this in order
     * to support (echo/noecho). We'll also do noncrmode input buffering here.
     * Maximum input line length is 132 columns.
     *
     * In nocrmode(), DOS functions are used to get a line and all the normal
     * command-line editing functions are available. Since there's no way to
     * turn off echo in this case, characters are echoed to the screen
     * regardless of the status of echo(). In order to retain control of the
     * window, input fetched for wgetch() is always done in crmode, even if
     * Crmode isn't set. If nl() mode is enabled, carriage return (Enter, ^M)
     * and linefeed (^J) are both mapped to '\n', otherwise they are not mapped.
     *
     * Characters are returned in an int. The high byte is 0 for normal
     * characters. Extended characters (like the function keys) are returned
     * with the high byte set to 0xff and the character code in the low byte.
     * Extended characters are not echoed.
     */

    static unsigned char  buf[ 133 ];	/* Assuming initialization to 0's */
    static unsigned char  *p = buf;
    register	    int	  c;

    if( !win->Crmode )
    {
		if( !*p ) p = getbuf( win, buf );
		return( win->Nl && *p == '\r' ) ? '\n' : *p++ ;
    }
    else
    {
		if(win->IO)
		{
			if( (c = fgetc((FILE *)win->IO) & 0xff) == ('Z'-'@') )	/* Ctrl-Z */
				return EOF ;
			else
			{
	    		if( c == '\r' &&  win->Nl ) c = '\n' ;
		    	if( win->Echo ) waddch( win, c );
			}
		}
		else
		{
			c = EOF;
		}
		return c;
    }
}

/*--------------------------------------------------------*/

int	waddch(WINDOW	*win,int c )
{
    /*	Print a character to an active (not hidden) window:
     *	The following are handled specially:
     *
     *	\n  Clear the line from the current cursor position  to the right edge
     *	    of the window. Then:
     *	        if nl() is active:
     *		    go to the left edge of the next line
     *	        else
     *		    go to the current column on the next line
     *	    In addition, if scrolling is enabled, the window scrolls if you're
     *	    on the bottom line.
     *	\t  is expanded into an 8-space field. If the tab goes past the right
     *      edge of the window, the cursor wraps to the next line.
     *	\r  gets you to the beginning of the current line.
     *	\b  backs up one space but may not back up past the left edge of the
     *      window. Nondestructive. The curses documentation doesn't say that
     *      \b is handled explicitly but it does indeed work.
     *	ESC This is not standard curses but is useful. It is valid only if
     *      DOESC was true during the compile. All characters between an ASCII
     *	    ESC and an alphabetic character are sent to the output but are other
     *      wise ignored. This let's you send escape sequences directly to the
     *      terminal if you like. I'm assuming here that you won't change
     *      windows in the middle of an escape sequence.
     *
     * Return ERR if the character would have caused the window to scroll
     * illegally, or if you attempt to write to a hidden window.
     */

    int		    rval    = OK;

    if( win->hidden )
	return ERR;

    cmove(win, win->y_org + win->row, win->x_org + win->col);

    {
	switch( c )
	{
	case '\b':  if( win->col > 0 )
		    {
		        outc(win, '\b', win->attrib  );
		        --( win->col );
		    }
		    break;

	case '\t':  do {
			waddch( win, ' ' );
		    } while( win->col % 8 );
		    break;

	case '\r':  win->col = 0;
		    cmove(win, win->y_org + win->row, win->x_org);
		    break;

	default:    
			if( (win->col + 1) < win->x_size )
		    {
				/* If you're not at the right edge of the window, */
				/* print the character and advance.		  */

				++win->col;
//				LockScreen();
				outc(win, c, win->attrib );
//				UnlockScreen();
				break;
		    }
		    replace(win, c);	  	  /* At right edge, don't advance */
		    if( !win->wrap_ok )
			{
//				UnlockScreen();
				break;
			}
//			UnlockScreen();

		    /* otherwise wrap around by falling through to newline */

	case '\n':  
//			LockScreen();
			if( c == '\n' )	   /* Don't erase character at far */
			{
				wclrtoeol( win );  /* right of the screen.	   */
			}

		    if( win->Nl )
				win->col = 0;
		    if( ++(win->row) >= win->y_size )
		    {
			rval = wscroll( win, 1 );
			--( win->row );
		    }
		    cmove(win, win->y_org + win->row, win->x_org + win->col);
//			UnlockScreen();
		    break;
	}
    }
    return rval;
}
