/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2007        */
/*-----------------------------------------------------------------------*/
/* This is a stub disk I/O module that acts as front end of the existing */
/* disk I/O modules and attach it to FatFs module with common interface. */
/*-----------------------------------------------------------------------*/

#include "diskio.h"
#include "Xmmc.h"

/*-----------------------------------------------------------------------*/
/* Correspondence between physical drive number and physical drive.      */
/* Note that Tiny-FatFs supports only single drive and always            */
/* accesses drive number 0.                                              */

#define ATA		0
#define MMC		1
#define USB		2


static DSTATUS DecodeMMCResult(unsigned char r)
{
	DSTATUS rV = 0;

	if(r & MMC_R1_BUSY) rV = STA_NOINIT;
	else if (r & (MMC_R1_PARAMETER | MMC_R1_ADDRESS | MMC_R1_ERASE_SEQ | MMC_R1_COM_CRC))
		rV = STA_NOINIT;
	else if (r & (MMC_R1_ILLEGAL_COM | MMC_R1_ERASE_RESET | MMC_R1_IDLE_STATE))
		rV = STA_NOINIT;
	return rV;
}

static DSTATUS DecodeMMCStatus(unsigned short r)
{
	DSTATUS rV;

	if(r & (MMC_R2_WP | MMC_R2_WP_ERASESKIPPED )) rV = STA_PROTECT;
	else if (r & ~(MMC_R2_WP | MMC_R2_WP_ERASESKIPPED) ) rV = STA_NOINIT;
	else rV = 0;
	return rV;
}

static DRESULT DecodeMMCIO(unsigned char r)
{
	DRESULT rV=0;

	if(r & MMC_R1_ILLEGAL_COM ) rV = RES_PARERR;
	else if (r & (MMC_R1_PARAMETER | MMC_R1_ADDRESS | MMC_R1_ERASE_SEQ | MMC_R1_COM_CRC))
		rV = RES_ERROR;
	else if (r & ( MMC_R1_ERASE_RESET | MMC_R1_IDLE_STATE | MMC_R1_BUSY))
		rV = RES_NOTRDY;
	return rV;
}

/*-----------------------------------------------------------------------*/
/* Inidialize a Drive                                                    */

DSTATUS disk_initialize (
	BYTE drv				/* Physical drive nmuber (0..) */
)
{
	DSTATUS stat = STA_NOINIT;
	int result;

	switch (drv) {
	case ATA :
//		result = ATA_disk_initialize();
		// translate the reslut code here

		return stat;

	case MMC :
		result = mmcReset();
		// translate the reslut code here
		stat = DecodeMMCResult(result);
		return stat;

	case USB :
//		result = USB_disk_initialize();
		// translate the reslut code here

		return stat;
	}
	return STA_NOINIT;
}



/*-----------------------------------------------------------------------*/
/* Return Disk Status                                                    */

DSTATUS disk_status (
	BYTE drv		/* Physical drive nmuber (0..) */
)
{
	DSTATUS stat=0;
//	unsigned short result;
//	int result;

	switch (drv) {
	case ATA :
//		result = ATA_disk_status();
		// translate the reslut code here

		return stat;

	case MMC :
//		result = MMC_disk_status();
		// translate the reslut code here
		stat = DecodeMMCStatus(mmcSendStatus());

		return stat;

	case USB :
//		result = USB_disk_status();
		// translate the reslut code here

		return stat;
	}
	return STA_NOINIT;
}



/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */

DRESULT disk_read (
	BYTE drv,		/* Physical drive nmuber (0..) */
	BYTE *buff,		/* Data buffer to store read data */
	DWORD sector,	/* Sector address (LBA) */
	BYTE count		/* Number of sectors to read (1..255) */
)
{
	DRESULT res = 0;
	unsigned char result;
	char i;


	switch (drv) {
	case ATA :
//		result = ATA_disk_read(buff, sector, count);
		// translate the reslut code here

		return res;

	case MMC :
//		result = MMC_disk_read(buff, sector, count);
		// translate the reslut code here
		for(i=0;(i<count) && (res == 0);++i)
		{
			result = mmcRead(sector++,&buff[((unsigned)i) << 9]);
			res = DecodeMMCIO(result);
		}

		return res;

	case USB :
//		result = USB_disk_read(buff, sector, count);
		// translate the reslut code here

		return res;
	}
	return RES_PARERR;
}



/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */

#if _READONLY == 0
DRESULT disk_write (
	BYTE drv,			/* Physical drive nmuber (0..) */
	const BYTE *buff,	/* Data to be written */
	DWORD sector,		/* Sector address (LBA) */
	BYTE count			/* Number of sectors to write (1..255) */
)
{
	DRESULT res = 0;
	unsigned char result;
	char i;

	switch (drv) {
	case ATA :
//		result = ATA_disk_write(buff, sector, count);
		// translate the reslut code here

		return res;

	case MMC :
//		result = MMC_disk_write(buff, sector, count);
		// translate the reslut code here
		for(i=0;(i<count) && (res == 0);++i)
		{
			result = mmcWrite(sector++,(unsigned char *)&buff[((unsigned)i) << 9]);
			res = DecodeMMCIO(result);
		}
		return res;

	case USB :
//		result = USB_disk_write(buff, sector, count);
		// translate the reslut code here

		return res;
	}
	return RES_PARERR;
}
#endif /* _READONLY */



/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */

DRESULT disk_ioctl (
	BYTE drv,		/* Physical drive nmuber (0..) */
	BYTE ctrl,		/* Control code */
	void *buff		/* Buffer to send/receive control data */
)
{
	DRESULT res=0;
//	int result;

	switch (drv) {
	case ATA :
		// pre-process here

//		result = ATA_disk_ioctl(ctrl, buff);
		// post-process here

		return res;

	case MMC :
		// pre-process here

//		result = MMC_disk_ioctl(ctrl, buff);
		// post-process here

		return res;

	case USB :
		// pre-process here

//		result = USB_disk_ioctl(ctrl, buff);
		// post-process here

		return res;
	}
	return RES_PARERR;
}

