//*********************************************************************************
// Device driver for Fat File System
//
// We use FatFS as an interface between the device driver and the SD Flash card
//
// Created Aug 12, 2008
//
//**********************************************************************************

#include <stdio.h>
#include <avr/io.h>
#include <cio.h>
#include "ff.h"
#include "FatFSDev.h"
#include "fcntl.h"
#include "Xmmc.h"

static int DOpen(IOCB *pIOCB) __attribute__((section(".lowtext")));
static int DClose(IOCB *pIOCB) __attribute__((section(".lowtext")));
static int DGetC(IOCB *pIOCB) __attribute__((section(".lowtext")));
static int DRead(IOCB *pIOCB,char *b, int n) __attribute__((section(".lowtext")));
static int DPutC(IOCB *pIOCB,int c) __attribute__((section(".lowtext")));
static int DWrite(IOCB *pIOCB,char *b, int n) __attribute__((section(".lowtext")));
static int DStatus(IOCB *pIOCB,int m) __attribute__((section(".lowtext")));
static int DXio(IOCB *pIOCB,int cmd,void *pP) __attribute__((section(".lowtext")));
static int DInit(void) __attribute__((section(".lowtext")));

static H_JVEC DJump = {
	DOpen,		//0
	DClose,		//1
	DGetC,		//2
	DRead,		//3
	DPutC,		//4
	DWrite,		//5
	DStatus,	//6
	DXio,		//7
	DInit		//8
};

static FATFS *FSYS;

static int Dmode(int mode)
{
	int retval = 0;

	if(mode & FCNTL_RDONLY)
		retval |= FA_READ;
	if(mode & FCNTL_WRONLY)
		retval |= FA_WRITE;
	return retval;
}

static int DOpen(IOCB *pIOCB)
{
	FF_DCB *pD = malloc(sizeof(FF_DCB));
	pIOCB->p = pD;
	pD->pF = malloc(sizeof(FIL));
	pIOCB->dev_name++;	//point just above device name
	pD->mode = Dmode(pIOCB->mode);
	return f_open(pD->pF,pIOCB->dev_name,pD->mode);
}

static int DClose(IOCB *pIOCB)
{
	f_close ((FIL *)pIOCB->p);
	free(pIOCB->p);		//free FIL object
	return 0;
}

static int DGetC(IOCB *pIOCB)
{
	return 0;
}

static int DRead(IOCB *pIOCB,char *b, int n)
{
	unsigned nread;

	f_read (((FF_DCB *)pIOCB->p)->pF,b,n,&nread);
	return (int)nread;
}

static int DPutC(IOCB *pIOCB,int c)
{
	return 0;
}

static int DWrite(IOCB *pIOCB,char *b, int n)
{
	return 0;
}

static int DStatus(IOCB *pIOCB,int m)
{
	return 0;
}

static int DXio(IOCB *pIOCB,int cmd,void *pP)
{
	return 0;
}

static int DInit(void)
{
	int retval = 0;

	FSYS = (FATFS *)malloc(sizeof(FATFS));
	if(FSYS)
	{
		if(f_mount(1,FSYS))
		{
			free(FSYS);
			FSYS = 0;
		}
	}
	CioAddHandler("D",&DJump);
	return retval;
}

void FatFSInit(void)
{
	//------------------------------------------------------------
	// This driver requires the use of the RTOS, so it cannot
	// be installed until after the OS has been started
	// (i.e. a call to StartOS()
	//------------------------------------------------------------
	DInit();
}
