/*@A (C) 1992 Allen I. Holub                                                */
#ifndef __BOX_H
#define __BOX_H
/*-------------------------------------------------------
 * BOX.H:  #defines for the box-drawing characters
 *-------------------------------------------------------
 *	The names are:
 *
 *	UL	Upper left corner
 *	UR	Upper right corner
 *	LL	lower left corner
 *	LR	lower right corner
 *	CEN	Center (intersection of two lines)
 *	TOP	Tee with the flat piece on top
 *	BOT	Bottom tee
 *	LEFT	Left tee
 *	RIGHT	Right tee
 *	HORIZ	Horizontal line
 *	VERT	Vertical line.
 *
 *			UL    -TOP-	UR 		HORIZ
 *				|
 *			L		 R		  V
 *		  	E	|	 I		  E
 *			F--   -CEN-    --G		  R
 *			T	|	 H		  T
 *			|		 T
 *				|
 *			LL    -BOT-	LR
 *
 * The D_XXX  defines have double horizontal and vertical lines.
 * The HD_XXX defines have double horizontal lines and single  vertical  lines.
 * The VD_XXX defines have double  vertical  lines and single horizontal lines.
 *
 * If your terminal is not IBM compatible, #define all of these as '+' (except
 * for the VERT #defines, which should be a |, and the HORIZ #defines, which
 * should be a -) by #defining NOT_IBM_PC before including this file.
 */

#define	VERT		0x92
#define	RIGHT		'+'
#define	UR		    0x99
#define	LL		    0x9a
#define	BOT		    '+'
#define	TOP		    0x93
#define	LEFT		'+'
#define	HORIZ		0x90
#define	CEN		    '+'
#define	LR		    0x9b
#define	UL		    0xa5
#define	D_VERT		'|'
#define	D_RIGHT		'+'
#define	D_UR		'+'
#define	D_LL		'+'
#define	D_BOT		'+'
#define	D_TOP		'+'
#define	D_LEFT		'+'
#define	D_HORIZ		'-'
#define	D_CEN		'+'
#define	D_LR		'+'
#define	D_UL		'+'
#define	HD_VERT		'|'
#define	HD_RIGHT	'+'
#define	HD_UR		'+'
#define	HD_LL		'+'
#define	HD_BOT		'+'
#define	HD_TOP		'+'
#define	HD_LEFT		'+'
#define	HD_HORIZ	'-'
#define	HD_CEN		'+'
#define	HD_LR		'+'
#define	HD_UL		'+'
#define	VD_VERT		'|'
#define	VD_RIGHT	'+'
#define	VD_UR		'+'
#define	VD_LL		'+'
#define	VD_BOT		'+'
#define	VD_TOP		'+'
#define	VD_LEFT		'+'
#define	VD_HORIZ	'-'
#define	VD_CEN		'+'
#define	VD_LR		'+'
#define	VD_UL		'+'

#endif /* __BOX_H */
