/*@A (C) 1992 Allen I. Holub                                                */
#ifndef __CURSES_H
#define __CURSES_H

#include "video.h"
#include "display.h"

/*!
 * \file CURSES.h
 * Simple Set of Text based windows routines
 *
 * \defgroup Curses Curses Windows Display
 *
 */

#ifndef max
#define max(a,b)	(((a) > (b)) ? (a) : (b))
#endif
#ifndef min
#define min(a,b)	(((a) < (b)) ? (a) : (b))
#endif

/*@{*/

/*! typedef for a Curses Window	*/
typedef struct _window_ WINDOW;

/*!
 * \struct _window_
 * descriptor for a Curses Window
 */
struct _window_
{
	int dvRow;		/*!< Current row for physical device (used by low level driver) */
	int dvCol;		/*!< Current column for physical device (used by low level driver)	*/
    int  x_org;	     		/*!< X coordinate of upper-left corner     */
    int  y_org;	     		/*!< Y coordinate of upper-left corner     */
    int  x_size;     		/*!< Horizontal size of text area.	 */
    int  y_size;     		/*!< Vertical   size of text area.	 */
    int  row;	     		/*!< Current cursor row    (0 to y_size-1) */
    int  col;	     		/*!< Current cursor column (0 to x_size-1) */
	void *IO;				/*!<	pointer to input channel			*/
    void *image;     			/*!< Image buffer. Holds what used to be   */
		     					/*!< under the window.			 */
    unsigned wrap_ok   :1 ; 	/*!< Line wrap is enabled in this win.     */
    unsigned scroll_ok :1 ; 	/*!< Scrolling permitted in this window    */
    unsigned hidden    :1 ; 	/*!< Window is hidden (nonstandard)        */
    unsigned boxed     :1 ; 	/*!< Window is boxed  (nonstandard)        */
	unsigned Echo	   :1 ;		/*!< Echo enabled			  */
	unsigned Crmode	   :1 ;		/*!< If 1, use buffered input	  */
	unsigned Nl		   :1 ;		/*!< If 1, map \r to \n on input	  */
	unsigned Save	   :1 ;		/*!< If 1, Window is save when overlapped	*/
    unsigned attrib    :8 ;		/*!< attribute used for character writes   */

};


#define	bool	unsigned int
#define	reg	register
#ifndef FALSE
#define	TRUE	(1)
#define	FALSE	(0)
#endif
#define	ERR	(0)
#define	OK	(1)

#define WIN_BOXED	1
#define WIN_UNBOXED	0

/*------------------------------------------------------------
 * The following macros implement many of the curses functions.
 */

#define getyx( win, y, x )  \
		    ((x) = ((WINDOW*)(win))->col, (y) = ((WINDOW*)(win))->row)

#define refresh()
#define scrollok(win,flag) ((win)->scroll_ok = (flag))
#define wrapok(win,flag)   ((win)->wrap_ok   = (flag))
#define wrefresh(win)	   /* empty */

/*----------------------------------------------------------
 * Nonstandard Macros: movewin() moves the window relative to the current
 * 		       position. Negative is left or up, positive is right or
 * down. ground() changes the fore and background colors for subsequent writes
 * to the window.
 */

#define mvwinr(w,dy,dx) mvwin((w),((w)->y_org - (w)->boxed) + (dy), \
				  ((w)->x_org - (w)->boxed) + (dx) )

#define ground(win,f,b)    ( win->attrib = ((f) & 0x7f) | ((b) & 0x7f) << 4)

/*---------------------------------------------------------
 * Externs for the window functions and #defines to map the standard screen
 * functions to the stdscr versions. There are a few idiosyncrasies here.
 * In particular, mvcur() just ignores it's first two arguments and maps to a
 * move() call. Similarly, subwin() just maps to a newwin() call, and clearok()
 * isn't supported. You must clear the window explicitly before writing to it.
 */

extern  WINDOW 		*stdscr;
extern	void endwin	(void);
extern  void initscr	(void);
extern	int waddch	(WINDOW *, int);

#define	addch(c)        waddch(stdscr, c)

extern	void waddstr	(WINDOW *, char *);
#define addstr(s)	waddstr(stdscr,s)

extern	void wclrtoeol	(WINDOW *	);
#define clrtoeol()	wclrtoeol(stdscr)

extern	void werase	(WINDOW *	);
#define erase		werase(stdscr)

#define wclear(w)	werase(w)
#define clear()		werase(stdscr)

extern	int wgetch	(WINDOW *);
#define getch()		wgetch( stdscr )

extern	void wmove	 (WINDOW *, int, int );
#define move(y,x)	 wmove( stdscr,(y),(x)  )
#define mvcur(oy,ox,y,x) move((y),(x))

extern	int wprintw	(WINDOW *, char *, ...);
extern	int printw	(          char *, ...);

extern  int wscroll	(WINDOW	*, int	       );
#define	scroll(win)	wscroll(win,1)

extern  int winch	(WINDOW *);
#define inch()		winch(stdscr)
#define mvinch(y,x)	( wmove(stdscr,y,x), winch(stdscr) )
#define mvwinch(w,y,x)  ( wmove(w,     y,x), winch(w)      )

extern	WINDOW *newwin	(int ,int ,int ,int, void *, int atrib, int nBoxed);
#define subwin(w,a,b,c,d)  newwin(a,b,c,d)

extern void WinSetTitle(WINDOW *w, char *title);

/*----------------------------------------------------------
 * Externs for functions that don't have stdscr versions
 */

extern void box		( WINDOW *,int ,int );		/* UNIX functions */
extern void crmode	( WINDOW *);
extern void delwin	( WINDOW * );
extern void echo	( WINDOW *);
extern  int mvwin	( WINDOW *win, int y, int x );
extern void nl		( WINDOW * );
extern void nocrmode	( WINDOW * );
extern void noecho	( WINDOW * );
extern void nonl	( WINDOW * );
extern void boxed	( WINDOW * );		/* Nonstandard functions */
extern void unboxed	( WINDOW * );
extern void save	( WINDOW * );
extern void nosave	( WINDOW * );
extern void def_ground	(WINDOW *, int, int );

/*@}*/

#endif /* __CURSES_H */
