#include "HeapManager.h"

/*!
 * \file HeapFreeSpace.c
 * \ingroup MultitaskingKernel
 */

/*@{*/

/*!
 * \fn size_t HeapFreeSpace(HEAP_BLOCK *pHeap)
 * \brief This function tells us how much free memory is available for allocation
 *
 * \param pHeap pointer to HEAP_BLOCK to check for free memory
 * \return number of bytes availiable for allocation
 */

size_t HeapFreeSpace(HEAP_BLOCK *pHeap)
{
	//----------------------------------
	// walk the free list and calculate
	// the number of free bytes
	//
	// A return value of 0 could mean
	// that malloc just hasn't been called
	// yet.
	//----------------------------------
	struct __freelist *pHF;
	size_t Free = 0;

	pHF = pHeap->__flp;

	while(pHF)
	{
		Free += pHF->sz;
		pHF = pHF->nx;
	}
	if (pHeap->BrkVal == 0)	pHeap->BrkVal = pHeap->Start;
	Free += (size_t)(pHeap->End - pHeap->BrkVal);
	return Free;
}

/*@}*/
