/****************************************************************************
** These are a collection of routines that implement a simple timer service
**
** Created May 14, 2008
**
****************************************************************************/

#include "task.h"
#include "TimerServices.h"
#include <stdlib.h>

ECB *TimerServicesSem;
TCB TTsrv;
static char TSname[] = "TSrvr";
static TSERVICE *TList;

THREAD(TimerServicesTask,arg)
{
	TSERVICE *pL;
	char sr;

	while(1)
	{
		PendSemaphore(TimerServicesSem,0);
		sr = Disable();
		pL = TList;
		while(pL)
		{
			pL->Counter--;
			if(pL->Counter == 0)
			{
				pL->Counter = pL->Time;
				(*pL->callback)(pL->arg);
			}
			pL = pL->next;
		}
		Enable(sr);
	}
}


TSERVICE *NewTimerService(unsigned t, unsigned m, void *arg, void (*cb)(void *) )
{
	//---------------------------------------------------------------------------
	// Create a new timer service
	//
	// parameters:
	//	t............Number of timer ticks before event generated
	//	m............Timer mode
	//	arg..........Pointer to argument to pass to call back function
	//	cb...........Pointer to call back function
	//
	// return value:
	//	returns pointer to timer service on success
	//	returns NULL on fail
	//----------------------------------------------------------------------------	
	TSERVICE *pTS = (TSERVICE *)malloc(sizeof(TSERVICE));
	if(pTS)
	{
		pTS->Time = t;
		pTS->Counter = t;
		pTS->Mode = m;
		pTS->arg = arg;
		pTS->callback = cb;
		pTS->next = 0;
		pTS->prev = 0;
	}
	return pTS;
}

void DeleteTimerService(TSERVICE *pTS)
{
	free(pTS);
}

void StartTimer(TSERVICE *pTS)
{
	//-------------------------------
	// Add pTS into a double linked
	// list....
	//-------------------------------
	char sr;
	sr = Disable();
	pTS->next = TList;
	if(TList)
		TList->prev = pTS;
	TList = pTS;
	Enable(sr);
}

void KillTimer(TSERVICE *pTS)
{
	char sr = Disable();
	if(pTS == TList)
	{
		TList = pTS->next;
		if(TList)
			TList->prev = 0;
	}
	else
	{
		if(pTS->prev)
			pTS->prev->next = pTS->next;
		if(pTS->next)
			pTS->next->prev = pTS->prev;
	}
	Enable(sr);
}

void InitTimerServices(void)
{
	TimerServicesSem = NewSemaphore(0,SEMAPHORE_MODE_BLOCKING,TSname);
	CreateTask(&TTsrv,TimerServicesTask,384,25,TSname,0);
	Insert(&ActiveTasks,&TTsrv);
	RegisterTickerPost(TimerServicesSem);
}


