// Plot.cpp : implementation file
//

#include "stdafx.h"
#include "Plot.h"
#include "WindowMessages.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPlot

CPlot::CPlot()
{
	BlackPen.CreatePen(PS_SOLID,1,COLORREF(0x00ff0000));
	Debug = 0;
	Data = 0;
	m_pSB = 0;
}

CPlot::~CPlot()
{
	delete[] Data;
}


BEGIN_MESSAGE_MAP(CPlot, CStatic)
	//{{AFX_MSG_MAP(CPlot)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_SCROLLBAR_LA,OnScrollBar)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPlot message handlers

void CPlot::AddPoint(int x, int y)
{
	if(nData < bsize)
	{
		Data[head] = y + MaxY/2;
		++head;
		++nData;
		if(head == bsize)head = 0;
	}
	else
	{
		++tail;
		if(tail == bsize) tail = 0;
		Data[head++] = y + MaxY/2;
		if(head == bsize) head = 0;
	}
	if(Debug)
	{
//		char *s = new char[256];
//		sprintf(s,"New Data = %d\n",y);
//		Debug->SetWindowText(s);
//		delete[] s;
	}
	Invalidate();
}

void CPlot::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	RECT clientrect;
	int x,y;
	int pos;

	GetClientRect(&clientrect);
	x = clientrect.right;
	y = clientrect.bottom;
	CBitmap BitmapPlot;		//this is where we will draw the histogram
	BitmapPlot.CreateCompatibleBitmap (&dc, x, y);

	CDC dcMem;
	dcMem.CreateCompatibleDC (&dc);

	
	CBrush brush (RGB (0, 255, 255));
	CBitmap* pOldBitmap = dcMem.SelectObject (&BitmapPlot);
	dcMem.FillRect (CRect (0, 0, x, y), &brush);
	CPen *OldPen = dcMem.SelectObject(&BlackPen);
	if(nData > 2)
	{
		int i=0;
		int index = tail;
		if(m_pSB)pos = m_pSB->pos;
		else pos = 0;
		char *s = new char[256];
		sprintf(s,"pos = %d",pos);
		Debug->SetWindowText(s);
		delete[] s;
//		if(pos+index > bsize)
//		{
//			if(Debug) Debug->SetWindowText("Index:Error");
//		}
//		else
			dcMem.MoveTo(i,ScaleY(Data[pos+index++]));
		if(index == bsize) index = 0;
		for(i=1;i<MaxX;++i)
		{
//			if(pos+index > bsize)
//			{
//				if(Debug) Debug->SetWindowText("Index:Error");
//			}
//			else
				dcMem.LineTo(i,ScaleY(Data[pos+index++]));
			if(index == bsize) index = 0;
		}
	}
	dc.BitBlt(0,0,x,y,&dcMem,0,0,SRCCOPY);
	dcMem.SelectObject(OldPen);
	dcMem.SelectObject (pOldBitmap);
	
	// Do not call CStatic::OnPaint() for painting messages
}

void CPlot::Test(CDisplayStatic *s)
{
	Debug = s;
}

void CPlot::InitPlot(int nSize)
{
	RECT clientrect;
	GetClientRect(&clientrect);

	if(nSize > 0)
	{
		Data = new int[nSize];
		bsize = nSize;
	}
	else
	{
		Data = new int[clientrect.right];
		bsize = clientrect.right;
	}
	head = 0;
	tail = 0;
	nData = 0;
	MaxY = clientrect.bottom;
	MaxX = clientrect.right;
}

void CPlot::AddScrollBar(CHorzScrollBar *pSB)
{
	m_pSB = pSB;
	m_pSB->SetActionMessage(m_hWnd,WM_SCROLLBAR_LA);
}

int CPlot::ScaleY(int nPy)
{
	return (int)((m_Scale_Gain * (double)nPy) - m_Scale_Offset);
}

void CPlot::SetScale(int MaxY, int MinY)
{
	RECT clientrect;
	GetClientRect(&clientrect);

	double X1,X2,Y1,Y2;

	X1 = (double)MaxY;
	X2 = (double)MinY;
	Y1 = (double)clientrect.bottom;
	Y2 = (double)clientrect.top;
	m_Scale_Gain = (Y1-Y2)/(X1-X2);	//slope
	m_Scale_Offset = (m_Scale_Gain * X1) - Y1;
	if(m_pSB)
	{
		m_pSB->SetScrollRange(0,bsize - clientrect.right);
	}
	if(Debug)
	{
		char *s = new char[256];
		sprintf(s,"m=%lf  b=%lf",m_Scale_Gain,m_Scale_Offset);
		Debug->SetWindowText(s);
		sprintf(s,"Y1=%lf  Y2 = %lf",Y1,Y2);
		Debug->SetWindowText(s);
		delete[] s;
	}
}

LRESULT CPlot::OnScrollBar(WPARAM w, LPARAM l)
{
	Invalidate();
	return 0;
}
