// SynthPort.cpp: implementation of the CSynthPort class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "synthconsol.h"
#include "SynthPort.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSynthPort::CSynthPort()
{
	State = MON_STATE_IDLE;
	pComP = 0;
	pBufferList = 0;
	pReleaseBuff = 0;
	m_NBuffers = 0;
	m_NRelease = 0;
	IncommingPayload = new char[1024];		//real big buffer
	MaxPayload = 1024;
}

CSynthPort::~CSynthPort()
{
	delete[] IncommingPayload;
}

int CSynthPort::ParseIncomingStream(int c)
{
	//**********************************************************
	// This function parses the character stream coming from
	// the acorn and executes the appropriate function
	// once it figures out what to do
	//**********************************************************
	AMSG *pMsg;

	switch(State)
	{
		case MON_STATE_IDLE:
			count.cV[0] = c;
			index = 0;
			State = MON_STATE_COLLECT_SIZE;
			break;
		case MON_STATE_COLLECT_SIZE:
			count.cV[1] = (char)c;
			State = MON_STATE_COLLECTCMD;
			break;
		case MON_STATE_COLLECTCMD:
			Cmd.cV[index++] = (char)c;
			if(index == 2)
			{
				index = 0;
				State = MON_STATE_COLLECTDATA;
			}
			break;
		case MON_STATE_COLLECTDATA:
			if(index < MaxPayload)
				IncommingPayload[index++] = (char)c;
			if(index == (count.sV - 2))
			{
				pMsg = CreateAMSG(index,IncommingPayload,Cmd.sV);
				pMsg->MsgCmd = Cmd.sV;
				//add message to message queue
				m_pMQ->Insert(pMsg);
				PostMessage(m_hWnd,m_ParseCompletMessage,0,0);
				State = MON_STATE_IDLE;
			}
			break;
	}
	return 0;
}

void CSynthPort::SetComPort(COverlappedSerial *pSP)
{
	pComP = pSP;
}

void CSynthPort::SetParseCompleteMessage(HWND nhWnd, int msg)
{
	m_hWnd = nhWnd;
	m_ParseCompletMessage = msg;
}

void CSynthPort::SetMessageQueue(CMsgQueue *pMSQ)
{
	m_pMQ = pMSQ;
}

void CSynthPort::SendTest(char *data)
{
	char *s = new char[256];
	int l;
	int i = 0;

	l = strlen(data) +1;
	s[i++] = 2+l;
	s[i++] = 0;
	s[i++] = MM_TEST;
	s[i++] = 0;
	memcpy(&s[i],data,l);
	pComP->WritePort(s,l+4);
	delete[] s;
}

void CSynthPort::SendLATrigger(char *tw)
{
	//------------------------------------
	// paramter
	//	tw points to an array of 8 chars
	//-------------------------------------
	char *s = new char[256];
	int i = 0,j;

	s[i++] = 10;
	s[i++] = 0;
	s[i++] = MM_SET_LA_TRIG;
	s[i++] = 0;
	for(j=0;j<8;++j) s[i++] = tw[j];
	pComP->WritePort(s,i);
	delete[] s;
}

void CSynthPort::SendSynthEnable(int enable)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_SYNTH_EN;
	s[i++] = 0;
	s[i++] = (char)enable;
	s[0] = i-2;		//packet size;
	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSynthSetOscFreq(int n, unsigned long f)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_SET_OSC_FREQ;
	s[i++] = 0;
	s[i++] = (char)LOW(n);		//channel number
	s[i++] = (char)MIDLO(n);
	s[i++] = (char)LOW(f);		//frequency
	s[i++] = (char)MIDLO(f);
	s[i++] = (char)MIDHI(f);
	s[i++] = (char)HIGH(f);
	s[0] = i-2;
	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSynthGetOscFreq(int n)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_GET_OSC_FREQ;
	s[i++] = 0;
	s[i++] = (char)LOW(n);		//channel number
	s[i++] = (char)MIDLO(n);
	s[0] = i-2;
	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

/*!
 * \fn void CSynthPort::SendSynthSetEv(int eg, int n,EVparams *pEV)
 * Function to set envelope generator parameters in synthesizer
 *
 * \param eg Envelope generator block (0 or 1)
 * \param n Envelope generator id (0->511)
 * \param pEV pointer to EVparams structure for setting up eg
 */

void CSynthPort::SendSynthSetEv(int eg, int n,EVparams *pEV)
{
	char *s = new char[256];	//buffer to store command in
	int i=0,j;						//byte counter variable
	char *pCEV = (char *)&pEV->Attack;

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_SET_EV;
	s[i++] = 0;
	s[i++] = (char)LOW(eg);		//envelope generator bank number
	s[i++] = 0;
	s[i++] = (char)LOW(n);		//envelope generator ID
	s[i++] = (char)MIDLO(n);
	for(j=0;j< sizeof(unsigned short)*4;++j,++i)
		s[i] = pCEV[j];
	s[0] = i-2;
	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSynthGetEV(int eg, int n)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_GET_EV;
	s[i++] = 0;
	s[i++] = (char)LOW(eg);		//envelope generator bank number
	s[i++] = (char)LOW(n);		//envelope generator ID
	s[i++] = (char)MIDLO(n);
	s[0] = i-2;
	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSynthSetFiltFreq(int n, unsigned short f)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSynthGetFiltFreq(int n)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSynthSetFiltQ(int n, unsigned short Q)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSynthGetFiltQ(int n)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendLAarm()
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_ARM_LA;
	s[i++] = 0;
	s[i++] = 0;
	s[0] = i-2;
	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendLAReadData(int n)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_READ_LA;
	s[i++] = 0;
	s[i++] = LOW(n);
	s[i++] = MIDLO(n);
	s[0] = i-2;
	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendLaReset()
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_RESET_LA;
	s[i++] = 0;
	s[i++] = 0;
	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendLaStatus(void)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_STATUS_LA;
	s[i++] = 0;
	s[i++] = 0;
	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendLaSetQualifier(unsigned short Qaul)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_SETQUAL_LA;
	s[i++] = 0;
	s[i++] = LOW(Qaul);
	s[i++] = MIDLO(Qaul);
	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendLaGetQualifier()
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_GETQUAL_LA;
	s[i++] = 0;
	s[i++] = 0;
	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendPeek(unsigned short adr)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_PEEK;
	s[i++] = 0;
	s[i++] = LOW(adr);
	s[i++] = MIDLO(adr);
	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSingleStep()
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_SINGLESTEP;
	s[i++] = 0;
	s[i++] = 0x0a;

	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendGetDebugData()
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_GETDEBUG;
	s[i++] = 0;
	s[i++] = 0x0a;

	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSetEvTrigger(int EvBank, int EvUnit, int v)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_EV_TRIGGER;
	s[i++] = 0;
	s[i++] = LOW(EvBank);
	s[i++] = MIDLO(EvBank);
	s[i++] = LOW(EvUnit);
	s[i++] = MIDLO(EvUnit);
	s[i++] = LOW(v);
	s[i++] = MIDLO(v);

	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up

}

void CSynthPort::SendGetEvStatus(int EvBank, int EvUnit)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_EV_STATUS;
	s[i++] = 0;
	s[i++] = LOW(EvBank);
	s[i++] = MIDLO(EvBank);
	s[i++] = LOW(EvUnit);
	s[i++] = MIDLO(EvUnit);

	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}

void CSynthPort::SendSingleStepSetChannel(unsigned short Ch)
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_SET_SSCHANNEL;
	s[i++] = 0;
	s[i++] = LOW(Ch);
	s[i++] = MIDLO(Ch);

	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up

}

void CSynthPort::SendSingleStepGetChannel()
{
	char *s = new char[256];	//buffer to store command in
	int i=0;						//byte counter variable

	s[i++] = 0;
	s[i++] = 0;
	s[i++] = MM_GET_SSCHANNEL;
	s[i++] = 0;
	s[i++] = 0;

	s[0] = i-2;

	pComP->WritePort(s,i);		//write data to COM port
	delete[] s;					//clean up
}
