`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    09:31:47 03/31/2008 
// Design Name: 
// Module Name:    ExtRam 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
// This module is used to interface up to the external ram on the Spartan3 Starter
// Board
// Created March 31, 2008 by Jim Patchell
//
//////////////////////////////////////////////////////////////////////////////////
module ExtRam(
    input [19:0] A,			//ram address
    input [7:0] Di,			//data in
    output reg [7:0] Do,			//Data out
    input W,					//write enable
    input R,					//read enable
    input [31:0] rDI,		//RAM interface DATA in
    output [31:0] rDO,		//RAM interface DATA out
    output [17:0] rAO,		//Ram Interface Address OUT
    output reg rCE1,				//ram interface CE1
    output reg rUB1,				//ram interface Upper Byte 1
    output reg rLB1,				//ram interface Lower Byte 1
    output reg rCE2,				//ram interface CE2
    output reg rUB2,				//ram interface Upper Byte 2
    output reg rLB2,				//ram interface Lower Byte 2
    output rWE,				//ram interface Write Enable
    output rOE					//ram Interface Read Enable
);

//-----------------------------------
// Data out mux...uses A0 and A1
//-----------------------------------
	always @ (rDI or A) begin
		case (A[1:0])
			2'b00:Do = rDI[7:0];
			2'b01:Do = rDI[15:8];
			2'b10:Do = rDI[23:16];
			2'b11:Do = rDI[31:24];
			default:Do = 8'h00;
		endcase
	end

	assign rDO[7:0] = Di;
	assign rDO[15:8] = Di;
	assign rDO[23:16] = Di;
	assign rDO[31:24] = Di;
	
//------------------------------------
// Byte Control Lines
//------------------------------------

	always @(A) begin
		case (A[1:0])
			2'b00: rLB1 = 0;	//active
			2'b01: rLB1 = 1;
			2'b10: rLB1 = 1;
			2'b11: rLB1 = 1;
			default: rLB1 = 1;
		endcase
	end

	always @(A) begin
		case (A[1:0])
			2'b00: rUB1 = 1;
			2'b01: rUB1 = 0;	//active
			2'b10: rUB1 = 1;
			2'b11: rUB1 = 1;
			default: rUB1 = 1;
		endcase
	end

	always @(A) begin
		case (A[1:0])
			2'b00: rLB2 = 1;
			2'b01: rLB2 = 1;
			2'b10: rLB2 = 0;	//active
			2'b11: rLB2 = 1;
			default: rLB2 = 1;
		endcase
	end

	always @(A) begin
		case (A[1:0])
			2'b00: rUB2 = 1;
			2'b01: rUB2 = 1;
			2'b10: rUB2 = 1;
			2'b11: rUB2 = 0;	//active
			default: rUB2 = 1;
		endcase
	end

	always @(A) begin
		case (A[1:0])
			2'b00: rCE1 = 0;	//active
			2'b01: rCE1 = 0;	//active
			2'b10: rCE1 = 1;
			2'b11: rCE1 = 1;	
			default: rCE1 = 1;
		endcase
	end

	always @(A) begin
		case (A[1:0])
			2'b00: rCE2 = 1;	
			2'b01: rCE2 = 1;	
			2'b10: rCE2 = 0;	//active
			2'b11: rCE2 = 0;	//active
			default: rCE2 = 1;
		endcase
	end

	assign rAO = A[19:2];	//output memory address
	assign rWE = !W;			//invert write signal
	assign rOE = !R;			//invert read signal
endmodule
