`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:49:14 02/08/2007 
// Design Name: 
// Module Name:    fifo16x8 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module fifo16x8(clk, reset, I, W, R, O, F, E, N);
    input clk;			//system clock
    input reset;		//reset
    input [7:0] I;	//data in
    input W;			//write strobe
    input R;			// read strobe
    output [7:0] O;	//output
    output F;		//full flag, 1 == FULL
    output E;		//empty flag, 1 == Empty
    output [3:0] N;	//number of bytes in fifo

	reg [7:0] O;
	wire F;
	wire E;
	
	//-------------------------------
	// dual port ram variables
	//-------------------------------
	
	wire [7:0] FDataO;
	reg Fwe;
	reg Fre;
	
	//-------------------------------
	// circular buffer pointers
	//-------------------------------
	wire [3:0] HeadAdr,TailAdr;
	reg  [3:0] N;

	always @ (posedge clk or posedge reset)
		if(reset)
			O <= 0;
		else if (Fre)
			O <= FDataO;

	always @ (HeadAdr or TailAdr)
		N = HeadAdr - TailAdr;
		
	assign F = (N == 4'b1111);
	assign E = (N == 4'b0000);

	always @ (N or W) begin
		if(N != 4'b1111)
			Fwe = W;
		else
			Fwe = 0;
	end
	
	always @ (N or R) begin
		if(N > 0)
			Fre = R;
		else
			Fre = 0;
	end
/*	
module dpr16x8(
	a,
	d,
	dpra,
	clk,
	we,
	spo,
	dpo);
*/
/*
dpr16x8 FifoRam(
	.a(HeadAdr),
	.d(I),
	.dpra(TailAdr),
	.clk(clk),
	.we(Fwe),
	.spo(),
	.dpo(FDataO)
);
*/
	
dpr16x8 FifoRam (
    .clk(clk), 
    .we(Fwe), 
    .D(I), 
    .A(HeadAdr), 
    .SPO(), 
    .DPRA(TailAdr), 
    .DPO(FDataO)
);

Cntr4 head (
    .clk(clk), 
    .reset(reset), 
    .ce(Fwe), 
    .Q(HeadAdr)
    );

Cntr4 tail (
    .clk(clk), 
    .reset(reset), 
    .ce(Fre), 
    .Q(TailAdr)
    );

endmodule
