`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    08:10:43 02/07/2007 
// Design Name: 
// Module Name:    muart 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Revision 0.10 -
//			Fixed typo in RxD instantiation where I had txd instead of rxd for
//			the input pin 9-25-2007
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module midiuart(clk, reset, D, O, A, wd, rd, I, rxd, txd);
    input clk;				//system clock
    input reset;			//system reset
    input [7:0] D;		//data bus in
    output [7:0] O;		//data bus out
    input [2:0] A;		//register select
    input wd;				//write data strobe
    input rd;				//read data strobe
    output  I;		//interrupts
	input rxd;				//uart input pin
	output txd;				//uart output pin
	
	reg [7:0] O;
	reg  I;
	wire txd;
	wire rxd;
	
parameter UART_DATA			=		3'b000;		//data IO address
parameter UART_RSTATUS 		=		3'b001;		//recieve status
parameter UART_TSTATUS		=		3'b010;		//transmit status
parameter UART_BRATE_L		=		3'b011;		//Baud Rate Low Byte
parameter UART_BRATE_H		=		3'b100;		//Baud Rate High Byte
parameter UART_IER			=		3'b101;		//interrupt enable register
parameter UART_IFR			=		3'b110;		//interrupt flag register
parameter UART_IVR			=		3'b111;

//-------------------------------------
// Interrupt Bits
//-------------------------------------

reg IRFifoHasChar;
reg ITFifoMT;
reg ITxdDone;

//-------------------------------------
// recieve fifo variables
//-------------------------------------

wire [3:0] RDCount;
wire wdRFifo;
wire [7:0] RFifoO;
wire rdRFifo;
wire REmpty,RFull;

//-------------------------------------
// Transmit fifo variables
//-------------------------------------

wire [3:0] TDCount;
wire [7:0] TFifoO;
reg wdTXD;
wire TEmpty,TFull;	//transmit fifo empty status
wire rdTFifo;	//read data from T fifo

//-------------------------------------
// Baud Rate Register Variables
//-------------------------------------

reg [7:0] BaudL,BaudH;	//baud rate registers
reg wdBL,wdBH;	//write strobes for baud rate register
wire bclk;	//bit rate clock

//--------------------------------------
// Recieve Uart Variables
//--------------------------------------

wire [7:0] RDout;
wire frame,overrun;

//---------------------------------------
// Transmit Uart Variables
//---------------------------------------

wire wdTxUart;	//write data into transmit uart
wire tbrate;	//transmit uart bit rate
wire done;		//status bit indicates transmit of char is done

//---------------------------------------
// State machine defines
//---------------------------------------
parameter TUART_STATE_IDLE = 2'b00;
parameter TUART_STATE_RDFIFO = 2'b01;
parameter TUART_STATE_WDUART = 2'b10;
parameter TUART_STATE_WAIT = 2'b11;

(* FSM_ENCODING="SEQUENTIAL", SAFE_IMPLEMENTATION="NO" *) reg [1:0] TState = TUART_STATE_IDLE; //state machine for transmit uart

//---------------------------------------
// Interrupt enable register variables
//---------------------------------------

reg [7:0] IER;
reg wdIER;

//---------------------------------------
// Interrupt Flag Register variables
//---------------------------------------

wire [7:0] IFR;
wire wdClrIFR;

parameter IFR_RXD     = 0;
parameter IFR_TXDMT   = 1;
parameter IFR_TXDDONE = 2;

//--------------------------------------------
// Interrupt Vector Register
//--------------------------------------------

reg [1:0] Ivr;

//**********************************************************************
// Methods Follow
//**********************************************************************

//-------------------------------------
// Interrupts
//-------------------------------------

always @ (posedge clk) begin
	//-------------------------------
	// If we read fifo and the fifo
	// is empty, then we clear
	// the interrupt
	//-------------------------------
	IRFifoHasChar <= ~REmpty;
end

always @ (posedge clk) ITFifoMT <= TEmpty;

always @ (posedge clk or posedge reset) begin
	if(reset)
		ITxdDone <= 0;		//init irq
	else if (TEmpty && done)
		ITxdDone <= 1;		//set IRQ
	else if (wdTXD)
		ITxdDone <= 0;		//clear IRQ
	else if (wdClrIFR && D[IFR_TXDDONE])
		ITxdDone <= 0;		//clear IRQ
end

assign IFR[IFR_RXD] = IER[IFR_RXD] & IRFifoHasChar;
assign IFR[IFR_TXDMT] = IER[IFR_TXDMT] & ITFifoMT;
assign IFR[IFR_TXDDONE] = IER[IFR_TXDDONE] & ITxdDone;
assign IFR[3] = 0;
assign IFR[4] = 0;
assign IFR[5] = 0;
assign IFR[6] = 0;
assign IFR[7] = 0;		//save for later


always @ (IFR) begin
	casex (IFR[2:0])
		3'b000:{I,Ivr} = 3'b100;	//no interrupt
		3'bxx1:{I,Ivr} = 3'b000;	//rxirq
		3'bx10:{I,Ivr} = 3'b001;	//txirq
		3'b100:{I,Ivr} = 3'b010;	//txdone irq
		default:{I,Ivr} = 3'b100;	//no interrupt
	endcase
end

//-------------------------------------
// write strobe decodes
//-------------------------------------

always @ (wd or A) begin
	if(wd && (A == UART_BRATE_L))
		wdBL = 1;
	else
		wdBL = 0;
end

always @ (wd or A) begin
	if(wd && (A == UART_BRATE_H))
		wdBH = 1;
	else
		wdBH = 0;
end

always @ (wd or A) begin
	if(wd && (A == UART_IER))
		wdIER = 1;
	else
		wdIER = 0;
end

always @ (wd or A) begin
	if(wd && (A == UART_DATA))
		wdTXD = 1;
	else
		wdTXD = 0;
end

assign wdClrIFR = wd & (A == UART_IFR);

//-------------------------------------
// Read Strobes
//
// A read strobe actually occures
// before a read operation...it is used
// to set things up for the actual
// read...for example...reading a FIFO
// is a clocked event, so we do a read
// strobe on the FIFO, and then we
// can actually see what is at the
// output of the FIFO with a uC.
//-------------------------------------

assign rdRFifo = rd & (A == UART_DATA);

//-------------------------------------
// Read Data MUX
//-------------------------------------

always @ (A or RFifoO or RDCount or TDCount or BaudL or BaudH or IER or REmpty or RFull or TEmpty or TFull or frame or overrun or IFR or Ivr) begin
	case (A)
		UART_DATA: O = RFifoO;
		UART_RSTATUS: O = {frame,overrun,REmpty,RFull,RDCount};
		UART_TSTATUS: O = {2'b0,TEmpty,TFull,TDCount};
		UART_BRATE_L: O = BaudL;
		UART_BRATE_H: O = BaudH;
		UART_IER:	  O = IER;
		UART_IFR:	  O = IFR;
		UART_IVR:	  O = {6'b000000,Ivr};
		default: O = 8'h7c;	//just some wierd value for debug
	endcase
end

//-------------------------------------
// Interrupt Enable Register
//-------------------------------------

always @ (posedge clk or posedge reset) begin
	if(reset)
		IER <= 0;	//disable all interrupts
	else if(wdIER)
		IER <= D;	//write data into IER
end

//-------------------------------------
// Baud Rate Registers
//-------------------------------------

always @ (posedge clk or posedge reset) begin
	if(reset)
		BaudL <= 0;
	else if (wdBL)
		BaudL <= D;	//write data into register
end

always @ (posedge clk or posedge reset) begin
	if(reset)
		BaudH <= 0;
	else if (wdBH)
		BaudH <= D;	//write data into register
end

//-------------------------------------
// recieve uart
//-------------------------------------



uartrx RxD (
    .dout(RDout), 
    .clk(clk), 
    .bclk(bclk), 
    .reset(reset), 
    .rxd(rxd),			 
    .frame(frame), 
    .overrun(overrun), 
    .ready(wdRFifo)
);

//-------------------------------------
// transmit uart
//-------------------------------------

uarttx TxD (
    .clock(clk), 
    .reset(reset), 
    .din(TFifoO), 
    .load(wdTxUart), 
    .brate(tbrate), 
    .txd(txd), 
    .done(done)
);

TBitRate TBitR (
    .clk(clk), 
    .reset(reset), 
    .sync(wdTxUart), 
    .R({BaudH,BaudL}), 
    .BR(tbrate)
);

   always@(posedge clk or posedge reset)
      if (reset) 
         TState <= TUART_STATE_IDLE;
      else
         (* FULL_CASE, PARALLEL_CASE *) case (TState)
            TUART_STATE_IDLE : begin
               if (!TEmpty)
                  TState <= TUART_STATE_RDFIFO;
               else
                  TState <= TUART_STATE_IDLE;
            end
            TUART_STATE_RDFIFO : begin
               TState <= TUART_STATE_WDUART;
            end
            TUART_STATE_WDUART : begin
               TState <= TUART_STATE_WAIT;
            end
            TUART_STATE_WAIT : begin
               if (done)
                  TState <= TUART_STATE_IDLE;
               else
                  TState <= TUART_STATE_WAIT;
            end
         endcase

   assign rdTFifo = (TState == TUART_STATE_RDFIFO);	//TUART_STATE_RDFIFO	
   assign wdTxUart =(TState == TUART_STATE_WDUART);	//TUART_STATE_WDUART

//-------------------------------------
// baud rate generator
//-------------------------------------

baudrategen BitRate (
    .clk(clk), 
    .reset(reset), 
    .R({BaudH,BaudL}), 
    .BR(bclk)
);

//-------------------------------------
// recieve fifo
//-------------------------------------

fifo16x8 RFifo (
    .clk(clk), 
    .reset(reset), 
    .I(RDout), 
    .W(wdRFifo), 
    .R(rdRFifo), 
    .O(RFifoO), 
    .F(RFull), 
    .E(REmpty), 
    .N(RDCount)
    );

//-----------------------------------------
// Transmit Fifo
//-----------------------------------------

fifo16x8 TFifo (
    .clk(clk), 
    .reset(reset), 
    .I(D), 
    .W(wdTXD), 
    .R(rdTFifo), 
    .O(TFifoO), 
    .F(TFull), 
    .E(TEmpty), 
    .N(TDCount)
    );

endmodule
