`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    14:02:46 09/14/2008 
// Design Name: 
// Module Name:    onepolefilter 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description:One pole filter for implementing various modules in a digital synth 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module onepolefilter(clk, reset, I, O, F, C, E);
    input clk;				//system clock
    input reset;			//system reset
    input [15:0] I;		//signal input
    output [15:0] O;		//signal output
	 input [15:0] F;		//frequency control of fileter section
    input [8:0] C;		//channel select bus
    input E;				//synth ennable

	//-------------------------------------
	// variables for Z element
	//-------------------------------------

wire [31:0] Zdata,Sum;
wire [33:0] FreqO;

reg [8:0] Cd1,Cd2;		//pipe delays for channel select
wire [16:0] DiffO;
reg [31:0] ZD1;
reg Ed1,Ed2;

always @ (posedge clk) begin
	Ed1 <= E;
	Ed2 <= Ed1;
end

always @ (posedge clk) begin	//pipe delay for channel select
	Cd1 <= C;
	Cd2 <= Cd1;
end
	
/*----------------------------
input clka;
input [31 : 0] dina;
input [8 : 0] addra;
input [0 : 0] wea;
output [31 : 0] douta;
input clkb;
input [31 : 0] dinb;
input [8 : 0] addrb;
input [0 : 0] web;
output [31 : 0] doutb;
----------------------------*/

dpram512x32 Zd(
	.clka(clk),
	.dina(Sum),
	.addra(Cd2),
	.wea(Ed2),
	.douta(),
	.clkb(clk),
	.dinb(0),
	.addrb(C),
	.web(1'b0),
	.doutb(Zdata)
);

always @ (posedge clk)	//pile line delay
	ZD1 <= Zdata;
	
/*--------------------
input [31 : 0] A;
input [31 : 0] B;
output [31 : 0] S;
---------------------*/

Adder32 Z(
	.A(FreqO[31:0]),
	.B(ZD1),
	.S(Sum)
);

// The following must be inserted into your Verilog file for this
// core to be instantiated. Change the instance name and port connections
// (in parentheses) to your own signal names.
 
MultS17x17 FreqCtl2 (
    .clk(clk),
    .a(DiffO), // Bus [16 : 0] 
    .b({1'b0,F}), // Bus [16 : 0] 
    .p(FreqO)); // Bus [33 : 0] 

/*
input [15 : 0] A;
input [15 : 0] B;
output [15 : 0] S;
A - B = S
*/

assign DiffO = {1'b0,I} - {1'b0,Zdata[31:16]};
/*
Subtractor16 Diff(
	.A(I),
	.B(Zdata[31:16]),
	.S(DiffO)
);
*/

assign O = Zdata[31:16];

endmodule
